@@ -48,8 +48,10 @@ def version():
         - order_by_order_type_ls修复为[""]的时候出现的bug
         - 页码p的提示更人性化, 改为必须正整数        # 2.4.6
         - 页码p或page_size等于GET_ALL(-999)时, 则返回全部数据
+        - admin增加orm检索      # 2.4.7
+        - simplrui的admin_class的基类改为`AjaxAdmin`
     """
-    v = "2.4.6"  # 当前: 2.4.6
+    v = "2.4.7"  # 当前: 2.4.7
     return v
 
 
