@@ -1,6 +1,7 @@
 """
 # BaseAdmin控制选项
 
+> 这里应该用装饰器模式的... 后续改进.
 > 导入功能要求用户必须拥有对应的add权限.
 
 - stop_field_ls                                         # 停用词列表, 字段名包含该词的字段将不展示.
@@ -15,6 +16,8 @@
         - export_asc = False                            # 按id升序导出
         - add_index = False                             # 导出时是否增加index列
 
+- orm_executor = True                                   # 使用orm过滤器
+
 - `ImportAdmin`导入导出选项
     - change_list_template = CHANGE_LIST_HTML_PATH      # 模板路径, 一般CHANGE_LIST_HTML_PATH的路径在'~bddjango/templates/entities/...'中
 
@@ -47,7 +50,7 @@ from ..pure import remove_temp_file
 
 import shutil
 # --- 初始化环境 ---
-from .admin_env_init import CHANGE_LIST_HTML_PATH, TEMPDIR, BD_USE_GUARDIAN, CHANGE_FORM_TEMPLATE
+from .admin_env_init import CHANGE_LIST_HTML_PATH, TEMPDIR, BD_USE_GUARDIAN, CHANGE_FORM_TEMPLATE, BD_USE_SIMPLEUI
 from tqdm import tqdm
 from pandas._libs.tslibs.timestamps import Timestamp
 from bddjango import get_base_model, get_model_max_id_in_db, old_get_model_max_id_in_db
@@ -57,9 +60,25 @@ from bdtime import tt
 import time
 import pandas as pd
 from django.contrib.auth import get_permission_codename
+import re
+import html
 
 
-class IDAdmin(admin.ModelAdmin):
+IAdmin = admin.ModelAdmin
+
+if BD_USE_SIMPLEUI:
+    from simpleui.admin import AjaxAdmin
+    IAdmin = AjaxAdmin
+
+if BD_USE_GUARDIAN:
+    from guardian.admin import GuardedModelAdminMixin
+
+    class IAdmin(IAdmin, GuardedModelAdminMixin):
+        change_form_template = CHANGE_FORM_TEMPLATE
+    # IAdmin = _IAdmin
+
+
+class IDAdmin(IAdmin):
     """
     * 保存时自动处理id, 解决postgre在批量导入数据后的主键冲突问题.
 
@@ -88,36 +107,6 @@ class IDAdmin(admin.ModelAdmin):
             obj.save()
 
 
-if BD_USE_GUARDIAN:
-    from guardian.admin import GuardedModelAdmin
-
-
-    class BaseGuarDianAdmin(GuardedModelAdmin):
-        """
-        * 保存时自动处理id, 解决postgre在批量导入数据后的主键冲突问题.
-
-        - 该方法仅对admin界面手动保存时调用的save_model()方法生效, 不影响obj.save()方法效率.
-        """
-        change_form_template = CHANGE_FORM_TEMPLATE
-
-        def save_model(self, request, obj, form, change):
-            """
-            参数change分辨保存or修改.
-            若为保存, 则model的id值自动更新为数据库中最大id+1.
-            """
-            try:
-                obj.save()
-            except Exception as e:
-                msg = f'可能为pgsql的id引起的错误:' + str(e)
-
-                print(msg)
-                reset_db_sequence(obj)
-                obj.save()
-
-
-    IDAdmin = BaseGuarDianAdmin
-
-
 # --- 处理df中的特殊格式
 def conv_date_field_str_format(ts):
     """
@@ -593,27 +582,6 @@ class ExcelImportExportAdmin(ImportAdmin, ExportExcelMixin):
     """
     default_export_action = True        # 默认增加导出按钮
 
-    def get_search_results(self, request, queryset, search_term):
-        """
-        * 若search_term以变量prefix的值开头, 则检索最近xx条记录.
-
-        - 如'~10'代表检索最近10条记录
-        """
-        prefix = '~'
-        if search_term is not None and not search_term.startswith(prefix):
-            ret = super().get_search_results(request, queryset, search_term)
-        else:
-            try:
-                if search_term.startswith(prefix):
-                    search_term = search_term
-                    search_term = int(search_term[len(prefix):])
-            except:
-                raise TypeError('id必须为整数!')
-            id_qsv_ls = queryset.values('id')[:search_term]
-            queryset = queryset.filter(id__in=id_qsv_ls)
-            ret = (queryset, False)
-        return ret
-
     def get_actions(self, request):
         ACTION_NAME = 'export_as_excel'
         if self.default_export_action and ACTION_NAME not in self.actions:
@@ -696,13 +664,6 @@ class ListDisplayAdmin(ExcelImportExportAdmin):
         return ret
 
 
-def list_display_admin_register(cls):
-    """
-    * 类装饰器, 未完待续.
-    """
-    return cls
-
-
 class ForceRunActionsAdmin(ListDisplayAdmin):
     """
     增加强制运行actions功能
@@ -748,6 +709,60 @@ class PureAdmin(admin.ModelAdmin):
         return super().changelist_view(request, extra_context)
 
 
-BaseAdmin = ForceRunActionsAdmin
+class BaseAdmin(ForceRunActionsAdmin):
+    """
+    # 若search_term以变量prefix的值开头, 则检索最近xx条记录.
+        - 如'~10'代表检索最近10条记录
 
+    # 支持orm检索
+    > 使用反引号[`]来替代引号["]
+        - .filter(id__lt=20, wiki_id="asd")
+        - .filter(wiki_id=`asd`).order_by(`-id`)[:3]
+    """
+    _tmp = None
+    orm_executor = True     # 使用orm过滤器
 
+    _ajax_search_term = None        # 用来ajax过滤
+
+    def get_search_results(self, request, queryset, search_term):
+        if not self.orm_executor:
+            ret = super().get_search_results(request, queryset, search_term)
+            return ret
+
+        prefix = '~'  # 反向过滤~pk
+
+        reg = re.compile(r'^\..*?[\)\]]$')  # 让.filter(xxx), .order_by(xxx)等orm语句可以执行
+
+        if hasattr(self, '_ajax_search_term') and getattr(self, '_ajax_search_term'):
+            search_term = self._ajax_search_term
+            self._ajax_search_term = None
+
+        if search_term and reg.match(search_term):
+            print('search_term: ', search_term)
+
+            # html转义
+            search_term = search_term.replace('`', '"')
+            while ('=&' in search_term):     search_term = html.unescape(search_term)
+            try:
+                self._tmp = queryset
+                exec(f'self._tmp = self._tmp{search_term}')
+                queryset = self._tmp
+                # exec(f'queryset = queryset{search_term}')     # 局部变量无法用exec赋值!
+            except Exception as e:
+                msg = f'orm语句执行出错! <br> search_term: {search_term},<br>  error: {e}'
+                self.message_user(request, msg)
+            ret = (queryset, False)
+        elif search_term.startswith(prefix):
+            try:
+                if search_term.startswith(prefix):
+                    search_term = search_term
+                    search_term = int(search_term[len(prefix):])
+            except Exception as e:
+                raise TypeError(f'pk必须为整数! {e}')
+            id_qsv_ls = queryset.values('pk')[:search_term]
+            queryset = queryset.filter(pk__in=id_qsv_ls)
+            ret = (queryset, False)
+        else:
+            ret = super().get_search_results(request, queryset, search_term)
+
+        return ret
